@@ -1,8 +1,8 @@
 /*  man.c: How to read and format man files.
-    $Id: man.c,v 1.12 2008/06/11 09:55:42 gray Exp $
+    $Id: man.c 5191 2013-02-23 00:11:18Z karl $
 
    Copyright (C) 1995, 1997, 1998, 1999, 2000, 2002, 2003, 2004, 2005, 
-   2007, 2008 Free Software Foundation, Inc.
+   2007, 2008, 2009, 2011, 2012 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,7 +20,9 @@
    Originally written by Brian Fox Thu May  4 09:17:52 1995. */
 
 #include "info.h"
+#ifndef __MINGW32__
 #include <sys/ioctl.h>
+#endif
 #include "signals.h"
 #if defined (HAVE_SYS_TIME_H)
 #include <sys/time.h>
@@ -53,7 +55,6 @@ static char const * const exec_extensions[] = { "", NULL };
 #endif
 
 static char *read_from_fd (int fd);
-static void clean_manpage (char *manpage);
 static NODE *manpage_node_of_file_buffer (FILE_BUFFER *file_buffer,
     char *pagename);
 static char *get_manpage_contents (char *pagename);
@@ -61,7 +62,8 @@ static char *get_manpage_contents (char *pagename);
 NODE *
 make_manpage_node (char *pagename)
 {
-  return info_get_node (MANPAGE_FILE_BUFFER_NAME, pagename);
+  return info_get_node (MANPAGE_FILE_BUFFER_NAME, pagename,
+                        PARSE_NODE_VERBATIM);
 }
 
 NODE *
@@ -105,7 +107,8 @@ get_manpage_node (FILE_BUFFER *file_buffer, char *pagename)
 	     the feet of info_windows[] array.  Therefore, all the
 	     nodes on that list which are showing man pages have their
 	     contents member pointing into the blue.  Undo that harm.  */
-	  if (old_contents && oldsize && old_contents != file_buffer->contents)
+	  if (old_contents && oldsize && old_contents != file_buffer->contents
+	      && info_windows)
 	    {
 	      int iw;
 	      INFO_WINDOW *info_win;
@@ -222,7 +225,9 @@ executable_file_in_path (char *filename, char *path)
 static char *
 find_man_formatter (void)
 {
-  return executable_file_in_path ("man", (char *)getenv ("PATH"));
+  char *man_command = getenv ("INFO_MAN_COMMAND");
+  return man_command ? man_command :
+                       executable_file_in_path ("man", getenv ("PATH"));
 }
 
 static char *manpage_pagename = NULL;
@@ -291,7 +296,7 @@ get_manpage_contents (char *pagename)
   if (manpage_section)
     formatter_args[arg_index++] = manpage_section;
   else
-    formatter_args[arg_index++] = "-a";    
+    formatter_args[arg_index++] = "-a";
 
   formatter_args[arg_index++] = manpage_pagename;
   formatter_args[arg_index] = NULL;
@@ -325,41 +330,34 @@ get_manpage_contents (char *pagename)
       freopen (NULL_DEVICE, "r", stdin);
       dup2 (pipes[1], fileno (stdout));
 
-      if (MB_CUR_MAX > 1)
-        {
-          /* Info has trouble wrapping man output if it contains
-             multibyte characters */
-          setenv("LANGUAGE", "C", 1);
-          setenv("LANG", "C", 1);
-#ifdef LC_MESSAGES
-          setenv("LC_MESSAGES", "C", 1);
-#endif
-          setenv("LC_CTYPE", "C", 1);
-          setenv("LC_ALL", "C", 1);
-        }
-
       execv (formatter_args[0], formatter_args);
 
       /* If we get here, we couldn't exec, so close out the pipe and
          exit. */
       close (pipes[1]);
-      xexit (0);
+      exit (EXIT_SUCCESS);
     }
 #else  /* !PIPE_USE_FORK */
   /* Cannot fork/exec, but can popen/pclose.  */
   {
     FILE *fpipe;
-    char *cmdline = xmalloc (strlen (formatter_args[0])
-			     + strlen (manpage_pagename)
-			     + (arg_index > 2 ? strlen (manpage_section) : 0)
- 			     + 3);
+    char *cmdline;
+    size_t cmdlen = 0;
     int save_stderr = dup (fileno (stderr));
     int fd_err = open (NULL_DEVICE, O_WRONLY, 0666);
+    int i;
+
+    for (i = 0; i < arg_index; i++)
+      cmdlen += strlen (formatter_args[i]);
+    /* Add-ons: 2 blanks, 2 quotes for the formatter program, 1
+       terminating null character.  */
+    cmdlen += 2 + 2 + 1;
+    cmdline = xmalloc (cmdlen);
 
     if (fd_err > 2)
       dup2 (fd_err, fileno (stderr)); /* Don't print errors. */
-    sprintf (cmdline, "%s %s %s", formatter_args[0], manpage_pagename,
-				  arg_index > 2 ? manpage_section : "");
+    sprintf (cmdline, "\"%s\" %s %s",
+	     formatter_args[0], formatter_args[1], formatter_args[2]);
     fpipe = popen (cmdline, "r");
     free (cmdline);
     if (fd_err > 2)
@@ -384,63 +382,6 @@ get_manpage_contents (char *pagename)
   return formatted_page;
 }
 
-static void
-clean_manpage (char *manpage)
-{
-  register int i, j;
-  int newline_count = 0;
-  char *newpage;
-
-  newpage = xmalloc (1 + strlen (manpage));
-
-  for (i = 0, j = 0; (newpage[j] = manpage[i]); i++, j++)
-    {
-      if (manpage[i] == '\n')
-        newline_count++;
-      else
-        newline_count = 0;
-
-      if (newline_count == 3)
-        {
-          j--;
-          newline_count--;
-        }
-
-      /* A malformed man page could have a \b as its first character,
-         in which case decrementing j by 2 will cause us to write into
-         newpage[-1], smashing the hidden info stored there by malloc.  */
-      if (manpage[i] == '\b' || (manpage[i] == '\f' && j > 0))
-        j -= 2;
-      else if (!raw_escapes_p)
-	{
-	  /* Remove the ANSI escape sequences for color, boldface,
-	     underlining, and italics, generated by some versions of
-	     Groff.  */
-	  if (manpage[i] == '\033' && manpage[i + 1] == '['
-	      && isdigit (manpage[i + 2]))
-	    {
-	      if (isdigit (manpage[i + 3]) && manpage[i + 4] == 'm')
-		{
-		  i += 4;
-		  j--;
-		}
-	      else if (manpage[i + 3] == 'm')
-		{
-		  i += 3;
-		  j--;
-		}
-	      /* Else do nothing: it's some unknown escape sequence,
-		 so let's leave it alone.  */
-	    }
-	}
-    }
-
-  newpage[j++] = 0;
-
-  strcpy (manpage, newpage);
-  free (newpage);
-}
-
 static NODE *
 manpage_node_of_file_buffer (FILE_BUFFER *file_buffer, char *pagename)
 {
@@ -470,6 +411,7 @@ manpage_node_of_file_buffer (FILE_BUFFER *file_buffer, char *pagename)
       node->parent   = NULL;
       node->flags = (N_HasTagsTable | N_IsManPage);
       node->contents += skip_node_separator (node->contents);
+      node->body_start = strcspn(node->contents, "\n");
     }
 
   return node;
@@ -568,8 +510,8 @@ find_reference_section (NODE *node)
 
   for (i = 0; reference_section_starters[i] != NULL; i++)
     {
-      position = search_forward (reference_section_starters[i], &frs_binding);
-      if (position != -1)
+      if (search_forward (reference_section_starters[i], &frs_binding,
+			  &position) == search_success)
         break;
     }
 
@@ -613,7 +555,7 @@ xrefs_of_manpage (NODE *node)
      within parenthesis. */
   reference_section->flags = 0;
 
-  while ((position = search_forward ("(", reference_section)) != -1)
+  while (search_forward ("(", reference_section, &position) == search_success)
     {
       register int start, end;
 
